#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/i2c-dev.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/ioctl.h>

#define COMM_PSEUDO_EEPROM	0x02
#define EEPROM_COMM_LEN		3
#define BMIC_I2C_DEV		"/dev/i2c-3"
#define BMIC_EEPROM_SLAVE_ADDR	0x15

int bmic_eeprom_init(void)
{
	int fd;
	int ret;

	fd = open(BMIC_I2C_DEV, O_RDWR);
	if (fd == -1)
		return -1;

	ret = ioctl(fd, I2C_SLAVE, BMIC_EEPROM_SLAVE_ADDR);
	if (ret == -1) {
		close(fd);
		return -1;
	}

	return fd;
}

static int bmic_eeprom_read_byte(int fd, int addr, unsigned char *val)
{
	unsigned char comm[EEPROM_COMM_LEN];

	comm[2] = (unsigned char) (addr >> 8);
	comm[1] = (unsigned char) addr;
	comm[0] = COMM_PSEUDO_EEPROM;

	if (write(fd, comm, EEPROM_COMM_LEN) != EEPROM_COMM_LEN)
		return -1;

	if (read(fd, val, 1) != 1) 
		return -1;

	return 0;
}

int bmic_eeprom_read(int fd, int addr, void *val, size_t len)
{
	unsigned char *buf = (unsigned char *)val;

	while (len--) {
		if (bmic_eeprom_read_byte(fd, addr, buf))
			return -1;
		addr++;
		buf++;
	}

	return 0;
}

void bmic_eeprom_exit(int fd)
{
	close(fd);
}
