/*
 * Copyright (C) 2010 Atmark Techno, Inc. All Rights Reserved.
 */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>

#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/socket.h>
#include <net/ethernet.h>

#include <openssl/rsa.h>
#include <openssl/bio.h>
#include <openssl/pem.h>
#include <openssl/evp.h>
#include <openssl/err.h>

#include <glib.h>

char public_key[] = "\
-----BEGIN PUBLIC KEY-----\n\
MIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEA+Xupbtw5WA2JYP88GNrZ\n\
OJIHRzudj3zilGtIsiZ/xmXSiwgIKWU6sGc1p8ShlhDd1tAsrfr74pLu7ewEabL9\n\
tnOrKOyXYP3Xx4bv10DCt0O6dvYKi/Bs9S98wGMTelF1bNR383xc12z9TIb7PiA1\n\
R5KyOUL2wpxsWWLT7Veckpy2Q7W0qx2CbslwMpS5LIiqk9x6XvC+rllN36fC3wsZ\n\
RWHK+gx7phOEIA6VQzeDbYuFQjcO6iMorzqucxvCRyvh0mmVoLJzVzzi4CxktWg5\n\
iRs3ApRNTpDPcupNUxzkUKjY/NPi6lmSVqdYRa/gbFIOOfdGJggGBUWJ7o/xHg2y\n\
q1MazL51088rD8XccO5wCEXxwlnTxjukuQE9z5vNoXuB28Z3aSeA12O/hJOrzKgS\n\
QlUMYme04yXqBZsQ74eQPdI14fTT50h35JTLYXwH79T3WKfHJF4ms2JW+Bz1d4H4\n\
Ui1XRf07uhKD5BsHWatfFlM9CJr/EHSaqJVYcRFXioV1qyJMNe48KaSMQd+BOQVR\n\
+Xovo+gWrBlXN44TGuRoIcJyfsdNkdr5/C7k6ZfZuiryxAq0OFeZFPHZhZkmIZAo\n\
NeWvMLOl3/MxDeyrK5SqM0vj9+SOhnQE/rZZM1EgWViiManibq4HXOsZ7R6Fi4M5\n\
Wxg6bC0FGyL8CA0yR5ARqc8CAwEAAQ==\n\
-----END PUBLIC KEY-----\n";

#include "get_info.h"

int main(void)
{
	unsigned char info[INFO_BUFFER_SIZE + SHA_DIGEST_LENGTH * 2];
	unsigned char *hash = &info[INFO_BUFFER_SIZE];
	unsigned char *sha1;
	BIO* bio;
	EVP_PKEY *pkey;
	RSA *rsa;
	unsigned char *encrypted;
	int ret;
	BIO *out;
	GString *basedir;
	int i;

	basedir = g_string_new(g_get_current_dir());
	if (!basedir) {
		g_printerr("unable to get current dir\n");
		return 1;
	}

	ret = get_info(info);
	if (ret) {
		g_printerr("unable to get board_info\n");
		return 1;
	}

	sha1 = SHA1(info, INFO_BUFFER_SIZE, NULL);
	for (i = 0; i < SHA_DIGEST_LENGTH; i++)
		sprintf((char *)&hash[i * 2], "%02x", sha1[i]);

	bio = BIO_new_mem_buf(public_key, sizeof public_key);
	if (!bio) {
		g_printerr("unable to allocate new bio for memory buffer\n");
		return 1;
	}

	pkey = PEM_read_bio_PUBKEY(bio, NULL, NULL, NULL);
	if (!pkey) {
		g_printerr("unable to read public key\n");
		return 1;
	}

	rsa = EVP_PKEY_get1_RSA(pkey);
	if (!rsa) {
		g_printerr("unable to get rsa\n");
		return 1;
	}

	if (RSA_size(rsa) < INFO_BUFFER_SIZE) {
		g_printerr("data buffer is too big\n");
		return 1;
	}

	encrypted = malloc(RSA_size(rsa));
	if (!encrypted) {
		g_printerr("unable to allocate data buffer\n");
		return 1;
	}

	ret = RSA_public_encrypt(INFO_BUFFER_SIZE + SHA_DIGEST_LENGTH * 2, info,
				 encrypted, rsa, RSA_PKCS1_PADDING);
	if (ret != RSA_size(rsa)) {
		g_printerr("unable to encode board info\n");
		return 1;
	}

	g_string_append(basedir, "/board-info.txt");
	out = BIO_new_file(basedir->str, "w+");
	if (!out) {
		g_printerr("unable to open output file\n");
		return 1;
	}

	ret = PEM_write_bio(out, "ATMARK TECHNO BOARD INFO",
			    "", encrypted, RSA_size(rsa));
	if (!ret) {
		g_printerr("unable to write to output file\n");
		g_debug("ret: %d\n", ret);
		return 1;
	}

	return 0;
}
