#include "get_serial.h"
#include "otp.h"
#include "get_mac_addr.h"
#include "get_uid.h"
#include "bmic_eeprom.h"

#define FORMAT			"[1]"
#define FORMAT_V2		"[2]"
#define FORMAT_SIZE		3
#define BOARDNAME_SIZE		7
#define BOARDNAME_LEN_SIZE	2
#define HEADER_SIZE (FORMAT_SIZE + BOARDNAME_LEN_SIZE + BOARDNAME_SIZE)
#define INFO_BUFFER_SIZE (HEADER_SIZE + KOTAI_SIZE + MAC_SIZE + UID_SIZE)

#define BI_BOARDTYPE	(0x20)

static uint32_t get_board_type(void)
{
	uint32_t val;
	int ret;
	int fd;

	fd = bmic_eeprom_init();
	if (fd == -1) {
		fprintf(stderr, "eeprom: can't access\n\n");
		return 0;
	}

	ret = bmic_eeprom_read(fd, EEPROM_BOARDTYPE_ADDR, &val, EEPROM_BOARDTYPE_LEN);
	if (ret) {
		fprintf(stderr, "eeprom: can't access\n\n");
		bmic_eeprom_exit(fd);
		return 0;
	}

	bmic_eeprom_exit(fd);

	return val;
}

static int get_info(unsigned char *buf)
{
	char *boardname;
	char *header = (char *)buf;
	char *serial = header + HEADER_SIZE;
	char *mac = serial + SERIAL_SIZE;
	char *uid = mac + MAC_SIZE;
	int ret;
	uint8_t serial_type;

	switch (get_board_type() & 0xffff0000) {
	case 0x0a000000:
		boardname = "AIOTG3_";
		break;
	case 0x0a100000:
		boardname = "AX1____";
		break;
	case 0x0a200000:
		boardname = "AX1L___";
		break;
	default:
		boardname = "UNKNOWN";
		break;
	}

	ret = get_serial_type(&serial_type);
	if (ret)
		return ret;

	if (serial_type == EEPROM_SERIAL_TYPE_KOTAI) {
		sprintf(header, "%s%02d%s", FORMAT_V2, BOARDNAME_SIZE, boardname);
		ret = get_kotai(serial);
		if (ret)
			return ret;

		mac = serial + KOTAI_SIZE;
	} else {
		sprintf(header, "%s%02d%s", FORMAT, BOARDNAME_SIZE, boardname);
		ret = get_serial(serial);
		if (ret)
			return ret;

		mac = serial + SERIAL_SIZE;
	}
	uid = mac + MAC_SIZE;

	ret = get_mac_addr(mac);
	if (ret)
		return ret;

	ret = get_uid(uid);
	if (ret)
		return ret;

	return 0;
}
