#include <stdio.h>
#include <stdint.h>
#include "bmic_eeprom.h"

#define BI_SERIAL	(0x28)
#define BI_SERIAL_V	(0x2f)

#define SERIAL_SIZE	(8)
#define KOTAI_SIZE	(12)

struct kotai_code {
	unsigned short product_id;
	unsigned short lot;
	unsigned short number;
};

int get_serial_type(uint8_t *type)
{
	int fd;
	int ret = 0;

	fd = bmic_eeprom_init();
	if (fd == -1) {
		fprintf(stderr, "eeprom: can't access\n\n");
		return -1;
	}

	ret = bmic_eeprom_read(fd, EEPROM_SERIAL_TYPE_ADDR, type, EEPROM_SERIAL_TYPE_LEN);
	if (ret) {
		fprintf(stderr, "eeprom: can't access\n\n");
		goto out;
	}

out:
	bmic_eeprom_exit(fd);

	return ret;
}

int get_serial(char *serial)
{
	int fd;
	int ret = 0;
	uint32_t buf;

	fd = bmic_eeprom_init();
	if (fd == -1) {
		fprintf(stderr, "eeprom: can't access\n\n");
		return -1;
	}

	ret = bmic_eeprom_read(fd, EEPROM_SERIALNO_ADDR, &buf, EEPROM_SERIALNO_LEN);
	if (ret) {
		fprintf(stderr, "eeprom: can't access\n\n");
		goto out;
	}
	sprintf(serial, "%08d", (int)buf);

out:
	bmic_eeprom_exit(fd);

	return ret;
}

int get_kotai(char *kotai_code)
{
	int ret = 0;
	int fd;
	struct kotai_code kotai;

	fd = bmic_eeprom_init();
	if (fd == -1) {
		fprintf(stderr, "eeprom: can't access\n\n");
		return -1;
	}

	ret = bmic_eeprom_read(fd, EEPROM_KOTAI_ADDR, &kotai, sizeof(kotai));
	if (ret) {
		fprintf(stderr, "eeprom: can't access\n\n");
		goto out;
	}
	sprintf(kotai_code, "%04X%04d%04d", (int)kotai.product_id, (int)kotai.lot, (int)kotai.number);

out:
	bmic_eeprom_exit(fd);

	return ret;
}
