
enum otp_type {
	OTP_UNKNOWN,
	OTP_FSL,
	OTP_NVMEM,
};
static enum otp_type otp_type = OTP_UNKNOWN;

#define NVMEM_PATH "/sys/bus/nvmem/devices/imx-ocotp0/nvmem"

static inline int otp_read(const char *file, uint32_t offset, uint32_t *data)
{
	int fd;
	char readbuf[32] = { 0 };

	if (otp_type == OTP_UNKNOWN) {
		if (!access(NVMEM_PATH, R_OK)) {
			otp_type = OTP_NVMEM;
		} else if (!access(file, R_OK)) {
			otp_type = OTP_FSL;
		} else {
			fprintf(stderr, "cannot find otp\n\n");
			return -1;
		}
	}

	if (otp_type == OTP_NVMEM)
		file = NVMEM_PATH;

	fd = open(file, O_RDONLY);
	if (fd < 0) {
		fprintf(stderr, "otp: can't open %s: %d\n\n", file, errno);
		return -1;
	}

	switch (otp_type) {
	case OTP_NVMEM:
		lseek(fd, offset, SEEK_SET);
		if (read(fd, data, sizeof(data)) != sizeof(data)) {
			fprintf(stderr, "otp: read error on %s\n", file);
			return -1;
		}
		break;
	case OTP_FSL:
		read(fd, readbuf, sizeof(readbuf)); // XXX short read etc
		*data = strtoul(readbuf, NULL, 0);
		break;
	default:
		abort();
	}

	return 0;
}
